#include <vtkCPPythonAdaptorAPI.h>
#include <vtkCPInputDataDescription.h>
#include <vtkCPProcessor.h>
#include <vtkLiveInsituLink.h>
#include <vtkCPPythonScriptPipeline.h>
#include <vtkCPDataDescription.h>
#include <vtkSMProxyManager.h>
#include <vtkSMSessionProxyManager.h>
#include <vtkSMProxy.h>
#include <vtkSMSourceProxy.h>
#include <vtkPVTrivialProducer.h>

#include <vtkUnstructuredGrid.h>
#include <vtkDoubleArray.h>
#include <vtkPoints.h>

#include <string>
#include <iostream>



void python(vtkUnstructuredGrid *grid) {

	std::string spath = "d:/build/CatalystTest/coproc.py";

	int tstep = 0, ctstep = 0;
	double time = 0;

	vtkCPPythonAdaptorAPI::CoProcessorInitialize(spath.c_str());
	vtkCPPythonAdaptorAPI::RequestDataDescription(&tstep, &time, &ctstep);
	vtkCPDataDescription *data = vtkCPPythonAdaptorAPI::GetCoProcessorData();

	data->GetInputDescriptionByName("input")->SetGrid(grid);
	data->SetForceOutput(true);

	vtkCPPythonAdaptorAPI::CoProcess();

	std::cin >> spath;								// Closing Paraview before continuing crashes the application on windows

	++tstep;
	time += 0.1;
	
	vtkCPPythonAdaptorAPI::RequestDataDescription(&tstep, &time, &ctstep);
	data->SetForceOutput(true);
	vtkCPPythonAdaptorAPI::CoProcess();

	std::cin >> spath;
	vtkCPPythonAdaptorAPI::CoProcessorFinalize();
}


void cpp(vtkUnstructuredGrid *grid) {

	double time = 0;
	vtkIdType tstep = 0;


	vtkCPProcessor *proc = vtkCPProcessor::New();
	proc->Initialize();
	vtkLiveInsituLink *link = vtkLiveInsituLink::New();
	link->SetInsituPort(22222);
	link->SetHostname("localhost");
	link->SetProcessType(vtkLiveInsituLink::INSITU);

	vtkSMSessionProxyManager *spxm = vtkSMProxyManager::GetProxyManager()->GetActiveSessionProxyManager();

	vtkSMProxy *px = spxm->NewProxy("sources", "PVTrivialProducer");


	vtkSMSourceProxy *spx = vtkSMSourceProxy::SafeDownCast(px);
	spxm->RegisterProxy("sources", spx);
	vtkObjectBase *obase = spx->GetClientSideObject();
	vtkPVTrivialProducer *prod =
		vtkPVTrivialProducer::SafeDownCast(obase);

	prod->SetOutput(grid,time);


	link->Initialize(spxm);
	spx->UpdatePipeline(time);
	link->InsituPostProcess(time, tstep);

	std::string tmp;

	std::cin >> tmp;								// Closing Paraview before continuing results in a crash

	link->InsituUpdate(time, tstep);
	spx->UpdatePipeline(time);
	link->InsituPostProcess(time, tstep);

	std::cin >> tmp;

	link->DropLiveInsituConnection();
}




int main() {


	vtkUnstructuredGrid* grid =
		vtkUnstructuredGrid::New();
	vtkPoints *points = vtkPoints::New();
	grid->SetPoints(points);
	grid->GetPoints()->InsertNextPoint(0, 0, 0);


	int i;

	std::cout << "insert 0 for python and 1 for cpp" << std::endl;

	std::cin >> i;

	switch (i)
	{
	case 0:
		python(grid);
	case 1:
		cpp(grid);
	default:
		break;
	}
	
}


